#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Author: thl-cmk[at]outlook[dot]com / thl-cmk.hopto.org
#
# Check Point base inventory
#
# 2016-06-29: inventory Check Point Appliance
# 2018-03-05: added Patches, Deployment Agent Build
# 2018-03-07: added Licenses
# 2020-06-01: cleanup, prepared for cmk1.7x, rename from inv_checkpoint_svn to checkpoint_inv_base
# 2020-11-27: rewrite for CMK check API 1.0 (CMK 2.0)
# 2020-11-28: added available updates
# 2021-02-08: transfered license/support info to seperate plugin
# 2021-02-22: code cleanup
# 2021-03-05: added hostlabel section
# 2021-03-18: fixed missing update agent snmp section.
# 2021-07-24: fixed parse function for empty string_table
#
# sample string_table
# [
#  [
#   ['5M7C043', 'Smart-1 5150', 'CheckPoint', 'Smart-1', 'R80.40', '994000022', 'Gaia', '3', '10']
#  ], 
#  [
#   ['1959']
#  ], 
# ]
#

from typing import List, NamedTuple, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    HostLabelGenerator,
    StringTable,
    InventoryResult,
)
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    Attributes,
    HostLabel,
    register,
    SNMPTree,
    startswith,
    all_of,
    any_of,
    equals,
)


def parse_inv_checkpoint_base(string_table: List[StringTable]) -> Optional[List]:
    class CheckpointInvBaseInfo(NamedTuple):
        serialnumber: str
        productname: str
        manufacturer: str
        series: str
        version: str
        build: str
        osname: str
        osmajorver: str
        osminorver: str

    if string_table == [[], []]:
        return

    section = []
    baseinfo = CheckpointInvBaseInfo(*string_table[0][0])

    try:
        updateagentversion = string_table[1][0][0]
    except IndexError:
        updateagentversion = ''

    invPath = ['hardware', 'system']

    for key, value in [
        ('serial', baseinfo.serialnumber),
        ('appliance_series', baseinfo.series),
        ('manufacturer', baseinfo.manufacturer),
        ('product_name', baseinfo.productname),
    ]:
        if not value == '':
            section.append((invPath, key, value))

    invPath = ['software', 'check_point', 'os_info']

    for key, value in [
        ('svn_version', baseinfo.version),
        ('svn_build', baseinfo.build),
        ('os_name', baseinfo.osname),
        ('deployment_agent_build', updateagentversion),
        ('os_version', baseinfo.osmajorver + '.' + baseinfo.osminorver),
    ]:
        if not value == '':
            section.append((invPath, key, value))

    return section


def host_label_inv_checkpoint_base(section: List) -> HostLabelGenerator:
    for invPath, key, value in section:
        if key == 'appliance_series' and value.lower() == 'maestro':
            yield HostLabel('checkpoint/device_type', 'maestro_security_group')


def inventory_checkpoint_base(section: List) -> InventoryResult:
    for invPath, key, value in section:
        yield Attributes(
            path=invPath,
            inventory_attributes={key: value})


register.snmp_section(
    name='inv_checkpoint_base',
    parse_function=parse_inv_checkpoint_base,
    host_label_function=host_label_inv_checkpoint_base,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.6',  # CHECKPOINT-MIB::svn
            oids=[
                '16.3',  # svnApplianceSerialNumber
                '16.7',  # svnApplianceProductName
                '16.9',  # svnApplianceManufacturer
                '16.10',  # svnApplianceSeries
                '4.1',  # svnVersion
                '4.2',  # svnBuild
                '5.1',  # osName
                '5.2',  # osMajorVer
                '5.3',  # osMinorVer
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.6.20',  # CHECKPOINT-MIB::svnUpdatesInfo
            oids=[
                '1',  # svnUpdatesInfoBuild
            ]
        ),
    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),  # sysObjectID == CheckPoint
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),  # sysObjectID == Linux
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),  # CheckPoint software installed
        )
    )
)

register.inventory_plugin(
    name='inv_checkpoint_base',
    inventory_function=inventory_checkpoint_base,
)
